CCBreaker = AceLibrary("AceAddon-2.0"):new("Parser-3.0","FuBarPlugin-2.0","AceConsole-2.0","AceDB-2.0")
local CCBabble  = AceLibrary("Babble-Spell-2.2")
local CCLocale  = AceLibrary("AceLocale-2.2"):new("CCBreaker")
local CCTablet = AceLibrary("Tablet-2.0")

CCBreaker.revision = tonumber(string.find("$Revision$","%d+")) or 1

-- Database

CCBreaker:RegisterDB("CCBreakerDB","CCBreakerDBPC")

-- basic datastructures

local oddline = {
    ['text']="",
    ['text2']="",
    ['text3']="",
    ['text4']="",
    ['textR']=1,
    ['textG']=1,
    ['textB']=1,
    ['text2R']=1,
    ['text2G']=1,
    ['text2B']=1,
    ['text3R']=1,
    ['text3G']=1,
    ['text3B']=1,
    ['text4R']=1,
    ['text4G']=1,
    ['text4B']=1,
    ['justify']="LEFT",
    ['justify2']="LEFT",
    ['justify3']="LEFT",
    ['justify4']="LEFT",
}

local evenline = {
    ['text']="",
    ['text2']="",
    ['text3']="",
    ['text4']="",
    ['textR']=0.7,
    ['textG']=0.7,
    ['textB']=0.7,
    ['text2R']=0.7,
    ['text2G']=0.7,
    ['text2B']=0.7,
    ['text3R']=0.7,
    ['text3G']=0.7,
    ['text3B']=0.7,
    ['text4R']=0.7,
    ['text4G']=0.7,
    ['text4B']=0.7,
    ['justify']="LEFT",
    ['justify2']="LEFT",
    ['justify3']="LEFT",
    ['justify4']="LEFT",
}

-- table recycling


function CCBreaker:recyclebreakertable()
	while #self.breakers > 0 do
		table.insert(self.breakertablecache, table.remove(self.breakers))
	end
end

function CCBreaker:getbreakertable()
	local t
	if #self.breakertablecache > 0 then
		t = table.remove(self.breakertablecache)
	else
		t = {}
	end
	return t
end

CCBreaker.breakers = {}
CCBreaker.breakertablecache = {}

-- Initialisation

CCBreaker.hasIcon = true

function CCBreaker:OnInitialize()
    self.defaultMinimapPosition = 300
end

function CCBreaker:OnEnable()
    self:RegisterParserEvent({eventType = "Dispel"},"broken")
end

function CCBreaker:OnDisable()
end

-- Workload

function CCBreaker:broken(info)
    if CCBreaker:filter(info) then
        self.lastinfo = info
        self:UpdateText()
        self.lastinfo = nil
        self:displayText(info)
    end
end


function CCBreaker:displayText(info)
    local hlcolor
    local basecolor
    local text
    local breaker = self:getbreakertable()
    
    if (info.recipientAbilityName) then 
        breaker.spell = info.recipientAbilityName
    else
        breaker.spell = ""
    end
    
    if (info.recipientName) then 
        breaker.target = info.recipientName
    else
        breaker.target = ""
    end
    
    if (info.sourceAbilityName) then 
        breaker.ability = info.sourceAbilityName
    else
        breaker.ability = ""
    end
    
    if (info.sourceName) then 
        breaker.breaker = info.sourceName
    else
        breaker.breaker = ""
    end
    table.insert(self.breakers,breaker)
    
    if self.db.char.display.text.colors then
        hlcolor = "|cffff8000"
        basecolor = "|cffffffff"
    else 
        hlcolor = ""
        basecolor = ""
    end
    if (self.db.char.display.text.console or self.db.char.display.text.center) then
        text = basecolor
        if info.sourceName then
          if info.sourceAbilityName then
              text = string.gsub(CCLocale["§s on §m was removed by §p's §a"],"§a",
                  hlcolor..info.sourceAbilityName..basecolor)
          else
              text = CCLocale["§s on §m was removed by §p"]
          end
          text = string.gsub(text,"§p",hlcolor..info.sourceName..basecolor)
        else
          text = CCLocale["§s on §m was removed"]
        end
        text = string.gsub(text,"§s",hlcolor..info.recipientAbilityName..basecolor)
        text = string.gsub(text,"§m",hlcolor..info.recipientName..basecolor)
        if self.db.char.display.text.console then
            self:Print(text)
        end
        if self.db.char.display.text.center then
            UIErrorsFrame:AddMessage(text, 1.0, 1.0, 1.0, 5.0)
        end
    end

end



function CCBreaker:filter(info)
    local sourceid
    local targetid
    if info.sourceID then
        sourceid = string.gsub(info.sourceID,"%d+","N")
    else 
        sourceid = nil
    end
    if info.recipientID then
        targetid = string.gsub(info.recipientID,"%d+","N")
    else
        targetid = nil
    end
    local spell    = info.recipientAbilityName
    return (
        -- "Show on" filter 
        (((targetid == "player") and (self.db.char.showon.player)) or
        ((targetid == "partyN") and (self.db.char.showon.party)) or
        ((targetid == "raidN") and (self.db.char.showon.raid)) or
        ((targetid == "pet") and (self.db.char.showon.pet)) or
        ((targetid == "partypetN") and (self.db.char.showon.partypet)) or
        ((targetid == "raidpetN") and (self.db.char.showon.raidpet)) or
        ((targetid == nil ) and (self.db.char.showon.others))) 
        and
        -- "Show From" filter
        (((sourceid == "player") and (self.db.char.showfrom.player)) or
        ((sourceid == "partyN") and (self.db.char.showfrom.party)) or
        ((sourceid == "raidN") and (self.db.char.showfrom.raid)) or
        ((sourceid == "pet") and (self.db.char.showfrom.pet)) or
        ((sourceid == "partapetN") and (self.db.char.showfrom.partypet)) or
        ((sourceid == "raidpetN") and (self.db.char.showfrom.raidpet)) or
        ((sourceid == nil) and (self.db.char.showfrom.others)))
        and
        -- "Show Effect" filter
        (((spell == CCBabble["Shackle Undead"]) and (self.db.char.showeffects.shackleundead)) or
        ((spell == CCBabble["Hibernate"]) and (self.db.char.showeffects.hibernate)) or
        ((spell == CCBabble["Sap"]) and (self.db.char.showeffects.sap)) or
        (((spell == CCBabble["Polymorph"]) or (spell == CCBabble["Polymorph: Pig"]) 
            or (spell == CCBabble["Polymorph: Turtle"])) and (self.db.char.showeffects.polymorph)) or
        ((spell == CCBabble["Freezing Trap Effect"]) and (self.db.char.showeffects.freezingtrap)))
    )   
end

function CCBreaker:OnTextUpdate()
    local text = nil
    local info = self.lastinfo
    if ((info == nil) or not
        (self.db.char.display.fubar.spell or 
        self.db.char.display.fubar.target or
        self.db.char.display.fubar.breaker or
        self.db.char.display.fubar.ability
        ))then 
        text = "CCBreaker"
    else
        if self.db.char.display.fubar.spell then 
            text = info.recipientAbilityName
        end
        if self.db.char.display.fubar.target then
            if text then text =  text .. "|" else text = "" end
            text = text..info.recipientName
        end
        if (self.db.char.display.fubar.breaker and info.sourceName) then
            if text then text =  text .. "|" else text = "" end
            text = text..info.sourceName
        end
        if (self.db.char.display.fubar.ability and info.sourceAbilityName) then
            if text then text =  text .. "|" else text = "" end
            text = text..info.sourceAbilityName
        end
    end 
        self:SetText(text)
end

function CCBreaker:OnTooltipUpdate()
    local cat = CCTablet:AddCategory(
        'text',"",
        'columns', 4
    )
    
    cat:AddLine(
        'text', CCLocale["Spell"],
        'text2',CCLocale["Target"],
        'text3',CCLocale["Breaker"],
        'text4',CCLocale["Ability"],
        'textR',0,
        'textG',1,
        'textB',0,
        'text2R',0,
        'text2G',1,
        'text2B',0,
        'text3R',0,
        'text3G',1,
        'text3B',0,
        'text4R',0,
        'text4G',1,
        'text4B',0,
        'justify',"LEFT",
        'justify2',"LEFT",
        'justify3',"LEFT",
        'justify4',"LEFT"
    )

    local isodd = true
    local current
    
    for i=1, #self.breakers do
        if isodd then current = oddline else current = evenline end
        isodd = not isodd
        current['text'] = self.breakers[#self.breakers+1-i].spell
        current['text2'] = self.breakers[#self.breakers+1-i].target
        current['text3'] = self.breakers[#self.breakers+1-i].breaker
        current['text4'] = self.breakers[#self.breakers+1-i].ability
        cat:AddLine(current)
    end
    
    CCTablet:SetHint("Rightclick for options")
    -- as a rule, if you have an OnClick or OnDoubleClick or OnMouseUp or OnMouseDown, you should set a hint.
end